#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2025.03.02 19:00:00                  #
# ================================================== #

MODE_AUDIO = "audio"
MODE_CHAT = "chat"
MODE_COMPLETION = "completion"
MODE_IMAGE = "img"
MODE_VISION = "vision"
MODE_ASSISTANT = "assistant"
MODE_LANGCHAIN = "langchain"
MODE_LLAMA_INDEX = "llama_index"
MODE_AGENT = "agent"
MODE_AGENT_LLAMA = "agent_llama"
MODE_EXPERT = "expert"
MODE_RESEARCH = "research"