pre code.hljs {
  display: block;
  overflow-x: auto;
  padding: 1em
}
code.hljs {
  padding: 3px 5px
}
/*

Pojoaque Style by Jason Tate
http://web-cms-designs.com/ftopict-10-pojoaque-style-for-highlight-js-code-highlighter.html
Based on Solarized Style from http://ethanschoonover.com/solarized

*/
.hljs {
  color: #dccf8f;
  background: url(./pojoaque.jpg) repeat scroll left top #181914
}
.hljs-comment,
.hljs-quote {
  color: #586e75;
  font-style: italic
}
.hljs-keyword,
.hljs-selector-tag,
.hljs-literal,
.hljs-addition {
  color: #b64926
}
.hljs-number,
.hljs-string,
.hljs-doctag,
.hljs-regexp {
  color: #468966
}
.hljs-title,
.hljs-section,
.hljs-built_in,
.hljs-name {
  color: #ffb03b
}
.hljs-variable,
.hljs-template-variable,
.hljs-title.class_,
.hljs-class .hljs-title,
.hljs-type,
.hljs-tag {
  color: #b58900
}
.hljs-attribute {
  color: #b89859
}
.hljs-symbol,
.hljs-bullet,
.hljs-link,
.hljs-subst,
.hljs-meta {
  color: #cb4b16
}
.hljs-deletion {
  color: #dc322f
}
.hljs-selector-id,
.hljs-selector-class {
  color: #d3a60c
}
.hljs-formula {
  background: #073642
}
.hljs-emphasis {
  font-style: italic
}
.hljs-strong {
  font-weight: bold
}