body {{ 
    color: #fff;
    background-color: #232629;
}}
::-webkit-scrollbar {{
    background: #232629;
}}
::-webkit-scrollbar-thumb {{
    background: {QTMATERIAL_SECONDARYLIGHTCOLOR};
}}
::-webkit-scrollbar-thumb:hover {{
    background: {QTMATERIAL_PRIMARYLIGHTCOLOR};
}}
::-webkit-scrollbar-corner {{
    background: #232629;
}}
a {{
    color: #a1b5c4 !important;
}}
a:hover {{
    color: #b8cad7 !important;
}}
.msg-user {{
    color: #cbcbcb !important;
}}
.msg-bot {{
    color: #fff;
}}
.msg-highlight {{
    border: 1px solid #323436;
    background-color: #272a2d;
}}
.cmd {{
    color: #4d4d4d;
}}
.ts {{
    color: #4d4d4d;
}}
.list {{
}}
code {{
    color: {QTMATERIAL_PRIMARYLIGHTCOLOR};
}}
.code-wrapper {{
    border: 1px solid #1d1f21;
}}
.code-header-wrapper {{
    background-color: #1d1f21;
}}
.code-header-copy {{
    color: #a0a0a0;
}}
.code-header-copy:hover {{
    color: #fff;
}}
.code-header-lang {{
    color: #686868;
}}
.action-icons a:hover {{
    filter: brightness(130%);
}}
.display-blocks {{
    background-color: #31363b;
}}
.display-blocks .msg-box {{
    background-color: #232629;
    border: 1px solid gray;
}}
.display-blocks .name-bot::before {{
    background-color: {QTMATERIAL_PRIMARYLIGHTCOLOR};
}}
.display-blocks .name-user::before {{
    background-color: silver;
}}